@@ -1,6 +1,8 @@
 from typing import List
 import abc
 
+from .task import Task
+
 
 class Engine(abc.ABC):
     """
@@ -11,6 +13,13 @@ class Engine(abc.ABC):
         pass
 
     @abc.abstractmethod
-    def execute(self, tasks: List):
+    def execute(self, tasks: List[Task]):
+        """Execute tasks.
+
+        Parameters
+        ----------
+        tasks:
+            List of tasks to execute.
+        """
         raise NotImplementedError(
             "This engine is not intended to be called.")
