@@ -1,13 +1,15 @@
 """
 Objective
 =========
-
 """
 
-from .objective import Objective
-from .amici_objective import AmiciObjective
+from .amici_calculator import AmiciCalculator
+from .amici import AmiciObjective, AmiciObjectBuilder
+from .function import Objective
 from .aggregated import AggregatedObjective
+from .priors import NegLogPriors, NegLogParameterPriors
 from .util import res_to_chi2, sres_to_schi2
+from .base import ObjectiveBase
 from .history import (
     HistoryOptions,
     HistoryBase,
@@ -17,9 +19,3 @@ from .history import (
     Hdf5History,
     OptimizerHistory)
 from . import constants
-
-# PEtab is an optional dependency
-try:
-    from .petab_import import PetabImporter
-except ModuleNotFoundError:
-    PetabImporter = None
