@@ -278,7 +278,7 @@ class AmiciObjective(ObjectiveBase):
     def check_mode(self, mode):
         return mode in [MODE_FUN, MODE_RES]
 
-    def call_unprocessed(self, x, sensi_orders, mode):
+    def call_unprocessed(self, x, sensi_orders, mode, edatas=None):
         sensi_order = max(sensi_orders)
 
         x_dct = self.par_arr_to_dct(x)
@@ -289,10 +289,12 @@ class AmiciObjective(ObjectiveBase):
             for data_ix in range(len(self.edatas)):
                 self.apply_steadystate_guess(data_ix, x_dct)
 
+        if edatas is None:
+            edatas = self.edatas
         ret = self.calculator(
             x_dct=x_dct, sensi_order=sensi_order, mode=mode,
             amici_model=self.amici_model, amici_solver=self.amici_solver,
-            edatas=self.edatas, n_threads=self.n_threads,
+            edatas=edatas, n_threads=self.n_threads,
             x_ids=self.x_ids, parameter_mapping=self.parameter_mapping,
             fim_for_hess=self.fim_for_hess,
         )
